#!/usr/bin/env node
"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = __importDefault(require("path"));
const fs_1 = __importDefault(require("fs"));
const lodash_1 = require("lodash");
const yargs_1 = __importDefault(require("yargs"));
const glob_1 = __importDefault(require("glob"));
const typescript_1 = __importDefault(require("typescript"));
function getBasePath() {
    const currentModuleRelativePath = lodash_1.get(require, 'main.filename');
    if (!currentModuleRelativePath)
        throw new Error('Unable to locate module path');
    const currentModuleAbsolutePath = path_1.default.dirname(currentModuleRelativePath);
    const pathComponents = currentModuleAbsolutePath.split('node_modules');
    const basePath = pathComponents.slice(0, pathComponents.length - 1).join('node_modules');
    return basePath;
}
function getConfig() {
    const args = yargs_1.default(process.argv).argv;
    const configFileName = args && args.config ? args.config : 'dsapidoc.json';
    const configFile = getBasePath() + configFileName;
    const config = JSON.parse(fs_1.default.readFileSync(configFile).toString());
    validateConfig(config);
    return config;
}
// function getCompilerOptions() {
//   const configFile = getBasePath() + 'tsconfig.json';
//   const { compilerOptions } = JSON.parse(fs.readFileSync(configFile).toString());
//   if(!compilerOptions) throw new Error('tsconfig.json/compilerOptions not found');
//   return compilerOptions;
// }
function validateConfig(config) {
    const { include, exclude } = config;
    if (!Array.isArray(include) || !Array.isArray(exclude))
        throw new Error('Malformed config file');
    [...include, ...exclude].forEach(e => {
        if (!/^\.\//.test(e))
            throw new Error('Paths must be relative to project root');
    });
}
function getFiles() {
    const { exclude, include } = getConfig();
    const ignore = exclude.map(item => getBasePath() + item.replace(/^\.\//, ''));
    const toInclude = include.reduce((all, item) => {
        const path = getBasePath() + item.replace(/^\.\//, '');
        const paths = glob_1.default.sync(path, { ignore });
        return [...new Set([...all, ...paths])];
    }, []);
    const filtered = toInclude.filter(e => e.endsWith('.ts') && !e.endsWith('.d.ts'));
    return filtered;
}
function run() {
    const files = getFiles();
    const compilerOptions = {};
    // getCompilerOptions();
    const program = typescript_1.default.createProgram(files, compilerOptions);
    const nodes = [];
    program.getSourceFiles().forEach(sf => {
        typescript_1.default.forEachChild(sf, node => {
            if (typescript_1.default.isModuleDeclaration(node)) {
                if ((node.flags & typescript_1.default.NodeFlags.Namespace) === 0 && node.body && typescript_1.default.isModuleBlock(node.body)) {
                    node.body.statements.forEach(statement => {
                        nodes.push(statement);
                    });
                    return;
                }
            }
            nodes.push(node);
        });
    });
    for (const node of nodes) {
        if (node.kind === typescript_1.default.SyntaxKind.ClassDeclaration && getDecorators(node, 'Controller').length > 0) {
            // for non string types consult tsoa
            const decorator = getDecorators(node, 'Controller')[0];
            const value = getDecoratorValues(decorator);
            console.log(value);
            const decorator2 = getDecorators(node, 'Tags')[0];
            const value2 = getDecoratorValues(decorator2);
            console.log(value2);
            const methods = node.members.filter((m) => m.kind === typescript_1.default.SyntaxKind.MethodDeclaration);
            for (const method of methods) {
                const decorator3 = getDecorators(method, 'Method')[0];
                const value3 = getDecoratorValues(decorator3);
                console.log(value3);
            }
        }
    }
}
run();
function getDecorators(node, name) {
    const decorators = node.decorators;
    if (!decorators || !decorators.length) {
        return [];
    }
    return decorators
        .filter((decorator) => decorator.expression.expression.escapedText === name);
}
// consult tsoa for non string values
function getDecoratorValues(decorator) {
    if (decorator && decorator.expression && Array.isArray(decorator.expression.arguments)) {
        return decorator.expression.arguments.map((e) => e.text);
    }
    return [];
}
;
